@@ -511,9 +511,10 @@ handle_directory (struct db *db, const struct db_header *hdr)
   return -1;
 }
 
-/* Read and handle DATABASE, open as FD */
+/* Read and handle DATABASE, opened as FD;
+   PRIVILEGED is non-zero if db_is_privileged() */
 static void
-handle_db (int fd, const char *database)
+handle_db (int fd, const char *database, bool privileged)
 {
   struct db db;
   struct db_header hdr;
@@ -530,6 +531,8 @@ handle_db (int fd, const char *database)
   if (db_read_name (&db, &path_obstack) != 0)
     goto err_path;
   obstack_1grow (&path_obstack, 0);
+  if (privileged == false)
+    hdr.check_visibility = 0;
   visible = hdr.check_visibility ? -1 : 1;
   p = obstack_finish (&path_obstack);
   if (handle_path (p, &visible) != 0)
@@ -921,13 +924,21 @@ finish_dbpath (void)
   conf_dbpath.entries = dest_path;
 }
 
+/* Drop set-group-ID privileges, if any. */
+static void
+drop_setgid (void)
+{
+  if (setgid (getgid ()) != 0)
+    error (EXIT_FAILURE, errno, _("can not drop privileges"));
+}
+
 /* Handle a conf_dbpath ENTRY, drop privileges when they are no longer
    necessary. */
 static void
 handle_dbpath_entry (const char *entry)
 {
   int fd;
-  bool keep_gid;
+  bool privileged_db;
 
   if (strcmp (entry, "-") == 0)
     {
@@ -936,12 +947,20 @@ handle_dbpath_entry (const char *entry)
 	       _("can not read two databases from standard input"));
       stdin_used = true;
       fd = STDIN_FILENO;
-      keep_gid = false;
+      privileged_db = false;
     }
   else
     {
       struct stat st;
 
+      if (stat (entry, &st) != 0)
+	{
+	  if (conf_quiet == false)
+	    error (0, errno, _("can not stat () `%s'"), entry);
+	  goto err;
+	}
+      if (!db_is_privileged (&st))
+	drop_setgid();
       fd = open (entry, O_RDONLY);
       if (fd == -1)
 	{
@@ -956,14 +975,11 @@ handle_dbpath_entry (const char *entry)
 	  close (fd);
 	  goto err;
 	}
-      keep_gid = db_is_privileged (&st);
+      privileged_db = db_is_privileged (&st);
     }
-  if (keep_gid == false)
-    {
-      if (setgid (getgid ()) != 0)
-	error (EXIT_FAILURE, errno, _("can not drop privileges"));
-    }
-  handle_db (fd, entry); /* Closes fd */
+  if (privileged_db == false)
+    drop_setgid();
+  handle_db (fd, entry, privileged_db); /* Closes fd */
  err:
   ;
 }
